module Prevoty
  class QueryAnalysis
    attr_reader :engine_version, :processed, :vendor, :vendor_version,
                :compliant, :statements, :error

    def initialize(data)
      @engine_version = data["engine_version"]
      @processed = data["processed"]
      @vendor = data["vendor"]
      @vendor_version = data["vendor_version"]
      @compliant = data["compliant"]
      @statements = data["statements"].map {|statement| Statement.new(statement)}
      @error = data["error"]
    end

    def to_json(options)
      return {
        engine_version: @engine_version, processed: @processed,
        vendor: @vendor, vendor_version: @vendor_version,
        compliant: @compliant, statements: @statements, error: @error
      }.to_json
    end
  end

  class MonitorQuery
    attr_reader :engine_version, :processed, :vendor, :vendor_version,
                :intelligence, :error

    def initialize(data)
      @engine_version = data["engine_version"]
      @processed = data["processed"]
      @vendor = data["vendor"]
      @vendor_version = data["vendor_version"]
      @intelligence = data["intelligence"].map {|statement| Intelligence.new(statement)}
      @error = data["error"]
    end

    def to_json(options)
      return {
        engine_version: @engine_version, processed: @processed,
        vendor: @vendor, vendor_version: @vendor_version,
        intelligence: @intelligence, error: @error
      }.to_json
    end
  end

  class Statement
    attr_reader :intelligence, :violations

    def initialize(data)
      @intelligence = Intelligence.new(data["intelligence"])
      @violations = Violations.new(data["violations"])
    end

    def to_json(options)
      return {
        intelligence: @intelligence, violations: @violations
      }.to_json
    end
  end

  class Intelligence
    attr_reader :statement_type, :sanitized_statement, :hashed_statement,
                :has_comment, :comparison_types, :row_creates, :column_reads,
                :column_updates, :row_deletes, :has_admin, :function_calls,
                :joins, :unions, :subqueries

    def initialize(data)
      @statement_type = data["statement_type"]
      @sanitized_statement = data["sanitized_statement"]
      @hashed_statement = data["hashed_statement"]
      @has_comment = data["has_comment"]
      @comparison_types = ComparisonTypes.new(data["comparison_types"])
      @row_creates = data["row_creates"].map {|el| Table.new(el)}
      @column_reads = data["column_reads"].map {|el| Column.new(el)}
      @column_updates = data["column_updates"].map {|el| Column.new(el)}
      @row_deletes = data["row_deletes"].map {|el| Table.new(el)}
      @has_admin = data["has_admin"]
      @function_calls = data["function_calls"].map {|el| FunctionCall.new(el)}
      @joins = data["joins"].map {|el| Table.new(el)}
      @unions = data["unions"].map {|el| Table.new(el)}
      @subqueries = data["subqueries"].map {|el| Table.new(el)}
    end

    def to_json(options)
      return {
        statement_type: @statement_type,
        sanitized_statement: @sanitized_statement,
        hashed_statement: @hashed_statement, has_comment: @has_comment,
        comparison_types: @comparison_types, row_creates: @row_creates,
        column_reads: @column_reads, column_upates: @column_updates,
        row_deletes: @row_deletes, has_admin: @has_admin,
        function_calls: @function_calls, joins: @joins, unions: @unions,
        subqueries: @subqueries
      }.to_json
    end
  end

  class ComparisonTypes
    attr_reader :contradiction, :dynamic, :static, :tautology

    def initialize(data)
      @contradiction = data["contradiction"]
      @dynamic = data["dynamic"]
      @static = data["static"]
      @tautology = data["tautology"]
    end

    def to_json(options)
      return {
        contradiction: @contradiction, dynamic: @dynamic, static: @static,
        tautology: @tautology
      }.to_json
    end
  end

  class Table
    attr_reader :database, :table

    def initialize(data)
      @database = data["database"]
      @table = data["table"]
    end

    def to_json(options)
      return {
        database: @database, table: @table
      }.to_json
    end
  end

  class Column
    attr_reader :database, :table, :column

    def initialize(data)
      @database = data["database"]
      @table = data["table"]
      @column = data["column"]
    end

    def to_json(options)
      return {
        database: @database, table: @table, column: @column
      }.to_json
    end
  end

  class FunctionCall
    attr_reader :name, :arguments

    def initialize(data)
      @name = data["name"]
      @arguments = data["arguments"]
    end

    def to_json(options)
      return {
        name: @name, arguments: @arguments
      }.to_json
    end
  end

  class Violations
    attr_reader :row_create_violations, :column_read_violations, :column_update_violations,
                  :row_delete_violation, :admin_violation, :union_violations, :join_violations,
                  :subquery_violations, :function_violations

    def initialize(data)
      @row_create_violations = data["row_create_violations"].map {|el| Table.new(el)}
      @column_read_violations = data["column_read_violations"].map {|el| Column.new(el)}
      @column_update_violations = data["column_update_violations"].map {|el| Column.new(el)}
      @row_delete_violations = data["row_delete_violations"].map {|el| Table.new(el)}
      @admin_violation = data["admin_violation"]
      @union_violations = data["union_violations"].map {|el| Table.new(el)}
      @join_violations = data["join_violations"].map {|el| Table.new(el)}
      @subquery_violations = data["subquery_violations"].map {|el| Table.new(el)}
      @function_violations = data["function_violations"].map {|el| FunctionCall.new(el)}
    end

    def to_json(options)
      return {
        row_create_violations: @row_create_violations,
        column_read_violations: @column_read_violations,
        column_update_violations: @column_update_violations,
        row_delete_violations: @row_delete_violations,
        admin_violation: @admin_violation, union_violation: @union_violation,
        join_violations: @join_violations, subquery_violations: @subquery_violations,
        function_violations: @function_violations
      }.to_json
    end
  end
end
